package ca.recrown.islandsurvivalcraft.world.generation.alternation;

import java.util.HashMap;
import java.util.Random;
import java.util.UUID;
import java.util.concurrent.locks.ReentrantReadWriteLock;
import java.util.concurrent.locks.ReentrantReadWriteLock.ReadLock;
import java.util.concurrent.locks.ReentrantReadWriteLock.WriteLock;

import org.bukkit.World;
import org.bukkit.plugin.java.JavaPlugin;

import ca.recrown.islandsurvivalcraft.world.generation.BiomeGenerator;
import ca.recrown.islandsurvivalcraft.world.generation.WorldGenerator;

/**
 * Alternates the data used on a per world basis.
 * Uses IslandWorldGenerator as the container for each world.
 */
public class WorldGeneratorAlternator {
    private final HashMap<UUID, WorldGenerator> chunkGenerators;
    private volatile WorldGenerator lastGenerator;
    private volatile UUID lastUUID;
    private final BiomeGenerator islandBiomeGenerator;
    private final ReentrantReadWriteLock lock = new ReentrantReadWriteLock(true);
    private final ReadLock readLock = lock.readLock();
    private final WriteLock writeLock = lock.writeLock();
    private final JavaPlugin plugin;

    public WorldGeneratorAlternator(JavaPlugin plugin, BiomeGenerator biomeGenerator) {
        this.chunkGenerators = new HashMap<>();
        this.islandBiomeGenerator = biomeGenerator;
        this.plugin = plugin;
    }

    public WorldGenerator getGenerator(World world, Random random) {
        if (lastUUID == null || (lastUUID.hashCode() != world.getUID().hashCode() && lastUUID.equals(world.getUID()))) {
            System.out.println("Alternating generator for world: " + world.getName());
            lastUUID = world.getUID();
            readLock.lock();
            try {
                lastGenerator = chunkGenerators.get(lastUUID);
            } finally {
                readLock.unlock();
            }
            if (lastGenerator == null) {
                lastGenerator = new WorldGenerator(plugin, world, islandBiomeGenerator, random);
                writeLock.lock();
                try {
                    chunkGenerators.put(lastUUID, lastGenerator);
                } finally {
                    writeLock.unlock();
                }
            }
        }
        return lastGenerator;
    }
}