package ca.recrown.islandsurvivalcraft.caching;

import static org.junit.Assert.assertSame;
import static org.junit.jupiter.api.Assertions.assertEquals;
import static org.junit.jupiter.api.Assertions.assertTrue;

import java.util.Random;

import org.junit.jupiter.api.AfterEach;
import org.junit.jupiter.api.BeforeAll;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.api.TestInstance;
import org.junit.jupiter.api.TestInstance.Lifecycle;

@TestInstance(Lifecycle.PER_CLASS)
public class CacheTest {
    Cache<Integer, String> integerCache;

    @BeforeAll
    public void setUp() {
        integerCache = new Cache<>(3);
    }

    @AfterEach
    public void cleanUp() {
        integerCache.clearCache();
    }

    @Test
    public void testBasicCaching() {
        String val = integerCache.getValue(0);
        assertTrue(val == null);
        val = "first";
        integerCache.setValue(0, val);
        assertTrue(integerCache.getValue(0) != null);
        assertSame(val, integerCache.getValue(0));

        val = integerCache.getValue(1);
        assertTrue(val == null);
        val = "second";
        integerCache.setValue(1, val);
        assertTrue(integerCache.getValue(1) != null);
        assertSame(val, integerCache.getValue(1));
    }

    @Test
    public void testUsageBasedClean() {
        integerCache.setValue(0, "first");
        integerCache.setValue(1, "second");
        integerCache.setValue(2, "third");
        
        assertEquals("first", integerCache.getValue(0));

        integerCache.setValue(3, "fourth");
        assertEquals("first", integerCache.getValue(0));

        integerCache.setValue(4, "fifth");
        assertTrue(integerCache.getValue(3) != null);
        assertTrue(integerCache.getValue(0) != null);
    }

    @Test
    public void testUsageLargeData() {
        int amount = 1024;
        Random random = new Random();

        Cache<Integer, Integer> largeCache = new Cache<>(amount/2);

        int[] values = new int[amount];
        for (int i = 0; i < amount; i++) {
            values[i] = random.nextInt();
            largeCache.setValue(i, values[i]);
        }

        for (int i = 0; i < amount /2; i++) {
            assertEquals(null, largeCache.getValue(i), "Current accessor: " + i);
        }

        for (int i = amount /2; i < amount; i++) {
            assertEquals(values[i], largeCache.getValue(i), "Current accessor: " + i);
        }
    }

    @Test
    public void testUsageLargeDataImportance() {
        int amount = 1024;
        Random random = new Random();

        Cache<Integer, Integer> largeCache = new Cache<>(amount/2);

        int[] values = new int[amount];
        for (int i = 0; i < amount; i++) {
            values[i] = random.nextInt();
            largeCache.setValue(i, values[i]);
            largeCache.getValue(0);
        }

        for (int i = 1; i < (amount/2) + 1; i++) {
            assertEquals(null, largeCache.getValue(i), "Current accessor: " + i);
        }

        for (int i = (amount /2) + 1; i < amount; i++) {
            assertEquals(values[i], largeCache.getValue(i), "Current accessor: " + i);
        }

        assertEquals(values[0], largeCache.getValue(0));
    }
}